# Copyright (c) 1997 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#      This product includes software developed by the Computer Systems
#      Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.


#=================================================
#                     Options
#=================================================

set opt(x)		100   
set opt(y)		100   
set opt(nn)		20;#5
set opt(stop)		150;#7
set opt(tr)		out.tr

#=================================================
#                      MAC
#=================================================

Mac/aob set dataRate_        2.0e6
Mac/aob set RTSThreshold_    3000

#=================================================
#                     Nodes
#=================================================

set ns_ [new Simulator]
set tracefd  [open $opt(tr) w]
$ns_ trace-all $tracefd

set topo   [new Topography]
$topo load_flatgrid $opt(x) $opt(y)
create-god  $opt(nn) 

$ns_ node-config -adhocRouting 	DumbAgent \
                 -llType 	LL \
                 -macType 	Mac/aob \
                 -ifqType 	Queue/DropTail \
                 -ifqLen 	50 \
                 -antType 	Antenna/OmniAntenna \
		 -propType 	Propagation/TwoRayGround \
                 -phyType 	Phy/WirelessPhy \
                 -channelType 	Channel/WirelessChannel \
                 -topoInstance 	$topo \
                 -agentTrace 	OFF \
                 -routerTrace 	OFF \
                 -macTrace 	ON \
		 -movementTrace OFF

for {set i 0} {$i < $opt(nn)} {incr i} {
    # WIRELESS TERMINAL instantiation 
    set WT($i) [ $ns_ node $i ]

    # setting the position for each node
    $WT($i) set X_ 10.0
    $WT($i) set Y_ 10.0
    $WT($i) set Z_ 0.0
}




#=================================================
#                     Agents
#=================================================
for {set i 0} {$i < $opt(nn)} {incr i 1} {
   set p($i) [new Agent/MAC_GEN_STATS $i]
   # associa o agent MAC_GENT_STATS ao nó
   $ns_ attach-agent $WT($i) $p($i)
   # associa o agent ao porto 290
   # porto do agent MAC_GENT_STATS   
   $WT($i) attach $p($i) 290  
}


for {set i 0} {$i < $opt(nn)} {incr i} {
   set mac_($i) [$WT($i) getMac 0];
   ## manter esta ordem de atribuiç\ao de parametrização
   $ns_ at 0.0000001 "$p($i) set_num_nodes $opt(nn)";
   $ns_ at 0.0000001 "$p($i) set_percent_broadcast 0.0";
   $ns_ at 0.0000001 "$p($i) set_mode 0";
   $ns_ at 0.0000001 "$p($i) set_lambda 8";
   #$ns_ at 0.0000001 "$p($i) set_scale 0.15";
   #$ns_ at 0.0000001 "$p($i) set_shape 150";
   # da BUG 20.0

   # logger instantiated at mac that is the same as $p() - static variable ...
   $ns_ at 0.00000000001 "$mac_($i) set_num_nodes $opt(nn)";
   $ns_ at 0.000000011 "$mac_($i) create_queue_logger";
   
   ## start sending at each agent
   $ns_ at 0.00000111 "$p($i) start";
#   $ns_ at 150.0 "$p($i) stop";
# print statistics ##$opt(stop)
   $ns_ at $opt(stop) "$p($i) print";
}    


#=================================================
#                     End
#=================================================

Agent/Ping instproc recv {from rtt} {
        $self instvar node_
        puts "node [$node_ id] received ping answer from \
              $from with round-trip-time $rtt ms."
}



$ns_ at $opt(stop).9 "$ns_ halt"

puts "Starting Simulation..."
$ns_ run
