/*********************************************************************************
 ** Universidade Nova de Lisboa - Faculdade de Ciências e Tecnologia
 **
 ** Protocolo MAC AOB - mac-aob-timers.h
 **
 ** Autores: 	
 ** Luís Lopes  
 ** Rodolfo Oliveira
 **  
 ** 20 de Outubro de 2009
 *********************************************************************************/

#ifndef __mac_aob_timers_h__
#define __mac_aob_timers_h__

/* ======================================================================
   Timers
   ====================================================================== */
class Mac_aob;

class aMacTimer : public Handler {
public:
	aMacTimer(Mac_aob* m) : mac(m) {
		busy_ = paused_ = 0; stime = rtime = 0.0;
	}

	virtual void handle(Event *e) = 0;

	virtual void start(double time);
	virtual void stop(void);
	virtual void pause(void) { assert(0); }
	virtual void resume(void) { assert(0); }

	inline int busy(void) { return busy_; }
	inline int paused(void) { return paused_; }
	inline double fs(void) { return fs_; }
	inline double su(void) { return su_; }
	inline void incsu(void) { su_++; }
	inline double time1(void) { return time1_; };
	inline double time2(void) { return time2_; };
	inline double numslots(void) { return numslots_; } 
	inline double expire(void) {
		return ((stime + rtime) - Scheduler::instance().clock());
	}

protected:
	Mac_aob	*mac;
	int		busy_;
	int		paused_;
	Event		intr;
	double		stime;	// start time
	double		rtime;	// remaining time
	double  	time1_;
	double  	time2_;
	bool		pausaesp;
	double  fs_;
	double  su_;
	double	numslots_;
};



class aBackoffTimer : public aMacTimer {
public:
	aBackoffTimer(Mac_aob *m) : aMacTimer(m), difs_wait(0.0) {}
	void	start(int cw, int idle, double difs = 0.0);
	void	handle(Event *e);
	void	pause(void);
	void	resume(double difs);
private:
	double	difs_wait;
};

class aDeferTimer : public aMacTimer {
public:
	aDeferTimer(Mac_aob *m) : aMacTimer(m) {}

	void	start(double);
	void	handle(Event *e);
};



class aIFTimer : public aMacTimer {
public:
	aIFTimer(Mac_aob *m) : aMacTimer(m) {}

	void	handle(Event *e);
};

class aNavTimer : public aMacTimer {
public:
	aNavTimer(Mac_aob *m) : aMacTimer(m) {}

	void	handle(Event *e);
};

class aRxTimer : public aMacTimer {
public:
	aRxTimer(Mac_aob *m) : aMacTimer(m) {}

	void	handle(Event *e);
};

class aTxTimer : public aMacTimer {
public:
	aTxTimer(Mac_aob *m) : aMacTimer(m) {}

	void	handle(Event *e);
};

#endif /* __mac_timers_h__ */

