/*********************************************************************************
 ** Universidade Nova de Lisboa - Faculdade de Ciências e Tecnologia
 **
 ** Protocolo MAC FCR - mac-fcr2-timers.cc
 **
 ** Autores: 	
 ** Luís Lopes  
 ** Rodolfo Oliveira
 **  
 ** 20 de Outubro de 2009
 *********************************************************************************/

#include <delay.h>
#include <connector.h>
#include <packet.h>
#include <random.h>
 
// #define DEBUG
//#include <debug.h>
#include <arp.h>
#include <ll.h>
#include <mac.h>
#include "mac-fcr2-timers.h"
#include "mac-fcr2.h" 

/*
 * Force timers to expire on slottime boundries.
 */
// #define USE_SLOT_TIME

// change wrt Mike's code

 #ifdef USE_SLOT_TIME
 #error "Incorrect slot time implementation - don't use USE_SLOT_TIME..."
 #endif

#define ROUND_TIME()	\
	{								\
		assert(slottime);					\
		double rmd = remainder(s.clock() + rtime, slottime);	\
		if(rmd > 0.0)						\
			rtime += (slottime - rmd);			\
		else							\
			rtime += (-rmd);				\
	}


/* ======================================================================
   Timers
   ====================================================================== */

void
f2MacTimer::start(double time)
{
	Scheduler &s = Scheduler::instance();
	assert(busy_ == 0);

	busy_ = 1;
	paused_ = 0;
	stime = s.clock();
	rtime = time;
	assert(rtime >= 0.0);


	s.schedule(this, &intr, rtime);
}

void
f2MacTimer::stop(void)
{
	Scheduler &s = Scheduler::instance();

	assert(busy_);

	if(paused_ == 0)
		s.cancel(&intr);

	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;
}

/* ======================================================================
   Defer Timer
   ====================================================================== */
void
f2DeferTimer::start(double time)
{
	Scheduler &s = Scheduler::instance();

	assert(busy_ == 0);

	busy_ = 1;
	paused_ = 0;
	stime = s.clock();
	rtime = time;
#ifdef USE_SLOT_TIME
	ROUND_TIME();
#endif
	assert(rtime >= 0.0);

	s.schedule(this, &intr, rtime);
}


void    
f2DeferTimer::handle(Event *)
{       
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;



	mac->deferHandler();
}


/* ======================================================================
   NAV Timer
   ====================================================================== */
void    
f2NavTimer::handle(Event *)
{       
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;

}


/* ======================================================================
   Receive Timer
   ====================================================================== */
void    
f2RxTimer::handle(Event *)
{       
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;

	mac->recvHandler();
}


/* ======================================================================
   Send Timer
   ====================================================================== */
void    
f2TxTimer::handle(Event *)
{       
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;



	mac->sendHandler();
}


/* ======================================================================
   Interface Timer
   ====================================================================== */
void
f2IFTimer::handle(Event *)
{
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;

	mac->txHandler();
}


/* ======================================================================
   Backoff Timer
   ====================================================================== */
void
f2BackoffTimer::handle(Event *)
{
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;
	difs_wait = 0.0;

	mac->backoffHandler();

}

void
f2BackoffTimer::start(int cw, int idle, double difs)
{
	int valor = 0;
	Scheduler &s = Scheduler::instance();

	assert(busy_ == 0);

	busy_ = 1;
	
	paused_ = 0;
	stime = s.clock();
	//Backoff in fcr2
	if (flag == 0)
	{
		valor = (Random::random() % cw);
		if (valor > 7 ){
			flag=1;
			rtime = 7* mac->phymib_.getSlotTime();
			rtime_total = valor * mac->phymib_.getSlotTime();
			rtime_falta = rtime_total - rtime;
		}
		else{
			rtime = valor * mac->phymib_.getSlotTime();
		}
	}
	else
	{
		rtime_total = rtime_falta/2;
		if (rtime_total < mac->phymib_.getSlotTime())
		{
			rtime=0.0;
			flag=0;
		}
		else
		{
			rtime = mac->phymib_.getSlotTime();
			rtime_falta = rtime_total - rtime;
		}
	}
#ifdef USE_SLOT_TIME
	ROUND_TIME();
#endif
	difs_wait = difs;

	
	
	assert(rtime + difs_wait >= 0.0);
	s.schedule(this, &intr, rtime + difs_wait);
}


void
f2BackoffTimer::stop()
{
	Scheduler &s = Scheduler::instance();
	
	
	flag = 0;
	busy_ = 0;
	paused_ = 0;
	stime = 0.0;
	rtime = 0.0;
	s.cancel(&intr);
}



