/*********************************************************************************
 ** Universidade Nova de Lisboa - Faculdade de Ciências e Tecnologia
 **
 ** Protocolo MAC FCR - mac-fcr2-timers.h
 **
 ** Autores: 	
 ** Luís Lopes  
 ** Rodolfo Oliveira
 **  
 ** 20 de Outubro de 2009
 *********************************************************************************/

#ifndef __mac_fcr2_timers_h__
#define __mac_fcr2_timers_h__

/* ======================================================================
   Timers
   ====================================================================== */
class Mac_fcr2;

class f2MacTimer : public Handler {
public:
	f2MacTimer(Mac_fcr2* m) : mac(m) {
		busy_ = paused_ = 0; stime = rtime = 0.0;
	}

	virtual void handle(Event *e) = 0;

	virtual void start(double time);
	virtual void stop(void);
	virtual void pause(void) { assert(0); }
	virtual void resume(void) { assert(0); }

	inline int busy(void) { return busy_; }
	inline int paused(void) { return paused_; }
	inline double expire(void) {
		return ((stime + rtime) - Scheduler::instance().clock());
	}
	
protected:
	Mac_fcr2	*mac;
	int		busy_;
	int		paused_;
	Event		intr;
	double		stime;	// start time
	double		rtime;	
	double		rtime_total; // remaining time total
	double		rtime_falta;
};



class f2BackoffTimer : public f2MacTimer {
public:
	f2BackoffTimer(Mac_fcr2 *m) : f2MacTimer(m), difs_wait(0.0) {}



	void	start(int cw, int idle, double difs = 0.0);
	void	handle(Event *e);
	void	stop(void);
	int flag;
private:
	double	difs_wait;
	
};

class f2DeferTimer : public f2MacTimer {
public:
	f2DeferTimer(Mac_fcr2 *m) : f2MacTimer(m) {}

	void	start(double);
	void	handle(Event *e);
};



class f2IFTimer : public f2MacTimer {
public:
	f2IFTimer(Mac_fcr2 *m) : f2MacTimer(m) {}

	void	handle(Event *e);
};

class f2NavTimer : public f2MacTimer {
public:
	f2NavTimer(Mac_fcr2 *m) : f2MacTimer(m) {}

	void	handle(Event *e);
};

class f2RxTimer : public f2MacTimer {
public:
	f2RxTimer(Mac_fcr2 *m) : f2MacTimer(m) {}

	void	handle(Event *e);
};

class f2TxTimer : public f2MacTimer {
public:
	f2TxTimer(Mac_fcr2 *m) : f2MacTimer(m) {}

	void	handle(Event *e);
};

#endif /* __mac_timers_h__ */

