/*********************************************************************************
 ** Universidade Nova de Lisboa - Faculdade de Ciências e Tecnologia
 **
 ** Protocolo MAC GDCF - mac-gdcf-timers.h
 **
 ** Autores: 	
 ** Luís Lopes  
 ** Rodolfo Oliveira
 **  
 ** 20 de Outubro de 2009
 *********************************************************************************/

#ifndef __mac_gdcf_timers_h__
#define __mac_gdcf_timers_h__

/* ======================================================================
   Timers
   ====================================================================== */
class Mac_gdcf;

class gdcfMacTimer : public Handler {
public:
	gdcfMacTimer(Mac_gdcf* m) : mac(m) {
		busy_ = paused_ = 0; stime = rtime = 0.0;
	}

	virtual void handle(Event *e) = 0;

	virtual void start(double time);
	virtual void stop(void);
	virtual void pause(void) { assert(0); }
	virtual void resume(void) { assert(0); }

	inline int busy(void) { return busy_; }
	inline int paused(void) { return paused_; }
	inline double expire(void) {
		return ((stime + rtime) - Scheduler::instance().clock());
	}

protected:
	Mac_gdcf	*mac;
	int		busy_;
	int		paused_;
	Event		intr;
	double		stime;	// start time
	double		rtime;	// remaining time
};



class gdcfBackoffTimer : public gdcfMacTimer {
public:
	gdcfBackoffTimer(Mac_gdcf *m) : gdcfMacTimer(m), difs_wait(0.0) {}



	void	start(int cw, int idle, double difs = 0.0);
	void	handle(Event *e);
	void	pause(void);
	void	resume(double difs);
private:
	double	difs_wait;
};

class gdcfDeferTimer : public gdcfMacTimer {
public:
	gdcfDeferTimer(Mac_gdcf *m) : gdcfMacTimer(m) {}

	void	start(double);
	void	handle(Event *e);
};



class gdcfIFTimer : public gdcfMacTimer {
public:
	gdcfIFTimer(Mac_gdcf *m) : gdcfMacTimer(m) {}

	void	handle(Event *e);
};

class gdcfNavTimer : public gdcfMacTimer {
public:
	gdcfNavTimer(Mac_gdcf *m) : gdcfMacTimer(m) {}

	void	handle(Event *e);
};

class gdcfRxTimer : public gdcfMacTimer {
public:
	gdcfRxTimer(Mac_gdcf *m) : gdcfMacTimer(m) {}

	void	handle(Event *e);
};

class gdcfTxTimer : public gdcfMacTimer {
public:
	gdcfTxTimer(Mac_gdcf *m) : gdcfMacTimer(m) {}

	void	handle(Event *e);
};

#endif /* __mac_timers_h__ */

